#!/usr/bin/ruby

require 'fileutils'
require_relative 'helper_utils'

PASCAL_REGEX_STRING = /\b[A-Z]+[a-zA-Z]*\b/

usage = <<-EOF
Please provide command and options

Usage:
scaffold Class <option>
scaffold Module <option>
scaffold MVC <option>
scaffold runner

options:
PascalCaseWord
EOF

# Parse ARGV
file_type = ARGV[0]
file_names = ARGV[1..-1].to_a

if (file_type.in ["Class", "Module", "MVC"]) && (file_names.any?)
  # Confirm all arguments are in PascalCasing
  if file_names.grep(PASCAL_REGEX_STRING) == file_names
    # Create 'spec' directory, if not exists
    FileUtils.mkdir_p 'spec'
    
    file_names.each do |file_name|
      # Convert file name to snake_casing
      snake_case = file_name.snakecase

      case file_type
      when "Class"
        # Create Class
        file_snippet = <<-EOF
class #{file_name}
end
EOF
        File.open("#{snake_case}.rb",  "a+") do |file|
          file.write(file_snippet)
        end

        # Create Class RSpec tests
        class_rspec_snippet = <<-EOF
require_relative '../#{snake_case}'

describe '#{file_name}' do
end
EOF
        File.open("spec/#{snake_case}_spec.rb",  "a+") do |file|
          file.write(class_rspec_snippet)
        end
      when "Module"
        # Create Module
        file_snippet = <<-EOF
module #{file_name}
end
EOF
        File.open("#{snake_case}.rb",  "a+") do |file|
          file.write(file_snippet)
        end

        # Create Module RSpec tests
        module_rspec_snippet = <<-EOF
require_relative '../#{snake_case}'

describe '#{file_name}' do
end
EOF
        File.open("spec/#{snake_case}_spec.rb",  "a+") do |file|
          file.write(module_rspec_snippet)
        end
      when "MVC"
        # Create Model
        model_snippet = <<-EOF
class #{file_name}
end
EOF
        File.open("#{snake_case}.rb",  "a+") do |file|
          file.write(model_snippet)
        end
        # Create Model RSpec tests
        model_rspec_snippet = <<-EOF
require_relative '../#{snake_case}'

describe '#{file_name} Model' do
end
EOF
        File.open("spec/#{snake_case}_spec.rb",  "a+") do |file|
          file.write(model_rspec_snippet)
        end
        # Create View
        view_snippet = <<-EOF
module #{file_name}View
end
EOF
        File.open("#{snake_case}_view.rb",  "a+") do |file|
          file.write(view_snippet)
        end
        # Create View RSpec tests
        view_rspec_snippet = <<-EOF
require_relative '../#{snake_case}_view'

describe '#{file_name} View' do
end
EOF
        File.open("spec/#{snake_case}_view_spec.rb",  "a+") do |file|
          file.write(view_rspec_snippet)
        end
        # Create Controller
        controller_snippet = <<-EOF
class #{file_name}Controller
end
EOF
        File.open("#{snake_case}_controller.rb",  "a+") do |file|
          file.write(controller_snippet)
          # Create Controller RSpec tests
          controller_rspec_snippet = <<-EOF
require_relative '../#{snake_case}_controller'

describe '#{file_name} Controller' do
end
EOF
        File.open("spec/#{snake_case}_controller_spec.rb",  "a+") do |file|
          file.write(controller_rspec_snippet)
        end
        end
      end # Confirm file type and presence of arguments
    end
  else
    puts usage
  end # Confirm regex
elsif (file_type.in ["runner"])
  runner_snippet = <<-EOF
require_relative ''
EOF
  File.open("runner.rb",  "a+") do |file|
    file.write(runner_snippet)
  end
else
  puts usage
end # Confirm arguments